// ==UserScript==
// @version        0.1
// @name           Gurooo
// @author         Emilien Klein
// @namespace      http://epicsys.com/
// @description    Enhances Guru by automatically giving the focus to the search field, and (planned feature) showing a list of saved contacts.
// @include        http://guru/
// @include        http://guru/Staff/Profile.aspx?id=*
// ==/UserScript==

/*

History:

08/30/2009 - v0.1 - First version of the script

*/

//unsafeWindow.console.log("XX");

var OOOUpdatesSent, OOOUpdatesReceived, OOOStatus;

function startProcess() {
    var url, onProfile, staffDetailsRight, newButton;
    // Check on which type of page we are
    url = window.location.href;
    onProfile = url.match("Profile.aspx");
    
    if (onProfile) { // We are on a profile page
        addSectionToProfile();
    }
    
    else { // We are on the main Guru page
        document.getElementById("ctl00_MasterPageHeader1_txtSearch").focus(); // Give the search bar the focus
        addSectionToGuruHome();
    }
}

function addSectionToProfile() {
    staffDetailsRight = getElementsByClass("Right")[1];
    contactStatus = createElement("div", {id: "contactStatus", style: "margin-top: 10px; display: none;"});
    staffDetailsRight.appendChild(contactStatus);
    // Check if this contact has already been saved
    savedContacts = deserialize("savedContacts");
    contactName = document.getElementById("ctl00_Content_ctlProfile_EmployeeIDCard_pnlCard_lblName").innerHTML;
    if (!savedContacts[contactName]) {
        // This is a new contact, display the button to add him/her to the list
        newButton = createElement("input", {value: "Save contact", type: "button", id: "saveContact", style: "margin-top: 10px;"});
        newButton.addEventListener("click", clickSaveContact, true);
        staffDetailsRight.appendChild(newButton);
    } else {
        // Tell the user that this contact has already been saved
        contactStatus.innerHTML = "<a href='' id='removeContact'>Remove " + contactName + " from your contacts list.</a>";
        document.getElementById("removeContact").addEventListener("click", clickRemoveContact, true);
        contactStatus.style.display = "block";
    }
}

function addSectionToGuruHome() {
    var searchBar, newHomeLinks, emptyListLink, newH2;
    
    // Add the container for the saved contacts
    // Select the searchBar div that is located on the left of the main Guru page
    searchBar = getElementsByClass("searchBar")[0];
    // Duplicate the first div to reuse the same styles etc. Remove the content from it.
    newHomeLinks = getElementsByClass("homeLinks")[0].cloneNode(true);
    remove(nextSibling(newHomeLinks.firstChild));
    // Insert a button that will empty the list
    //insertAtBeginning(createElement("input", {type: "button", id: "emptyListLink", value:"Empty your list", style: "width: 100px; font-size: 80%; margin-top: 10px; display: none;"}), newHomeLinks);
    emptyListLink = createElement("a", {href: "", id: "emptyListLink", style: "width: 100px; font-size: 80%; margin-top: 10px; display: none;"});
    emptyListLink.innerHTML = "Empty your list";
    insertAtBeginning(emptyListLink, newHomeLinks);
    // Insert a div that will contain the saved contacts
    insertAtBeginning(createElement("div", {id: "savedContacts"}), newHomeLinks);
    // Add a title to this section
    newH2 = createElement("h2", {style: "margin-left: 10px;"});
    newH2.innerHTML = "Saved contacts";
    insertAtBeginning(newH2, newHomeLinks);
    // Add the whole new section back to the page
    insertAtBeginning(newHomeLinks, searchBar);
    
    // Populate the saved contacts section
    populateSavedContacts();
}

function populateSavedContacts() {
    var savedContacts, upToDateOOOInfo, savedContactsContent, OOOStat;
    // Get the saved contacts
    savedContacts = deserialize("savedContacts");
    if (savedContacts.__count__) {
        // Check if the OOO informations is up to date
        upToDateOOOInfo = checkOOO(savedContacts);
        savedContactsContent = "";
        OOOStat = deserialize("OOOStatus");
        for (var i in savedContacts){
            // The contact's name and a link to his profile
            savedContactsContent += "<a href='Staff/Profile.aspx?id=" + savedContacts[i][0] + "' style='display: inline;'>" + i + "</a> ";
            // The contact's phone number.
            savedContactsContent += "(" + savedContacts[i][1] + ") ";
            // Check if the contact is OOO
            savedContactsContent += "<img class='IconImg' id='OOO_" + savedContacts[i][0] + "' src='/Images/icons/OutOfOffice.gif'";
            if (!upToDateOOOInfo || !(savedContacts[i][0] in oc(OOOStat))) {
                savedContactsContent += " style='display: none;'";
            }
            savedContactsContent += " /> ";
            // The contact's email address.
            savedContactsContent += "- <a href='" + savedContacts[i][2] + "' style='display: inline; border: none; padding: 0px;'>E-mail</a><br />";
        }
        document.getElementById("emptyListLink").style.display = "block";
        document.getElementById("emptyListLink").addEventListener("click", clickEmptyList, true);
    } else {
        savedContactsContent = "Your list of saved contacts is <em>empty</em>.";
        if(GM_getValue("savedContacts_backup")){
            savedContactsContent += " However, we have <strong>backup data available</strong> to <a href='' id='restoreContacts' style='display: inline; border: none; padding: 0px;'>restore your previously saved contacts</a>.<br />";
        }
        savedContactsContent += "<br />";
        savedContactsContent += "Press the <strong>Save contact</strong> button on a colleague's Guru profile to add this person to your list.<br /><br />";
        savedContactsContent += "If you need more help, please go to: <strong><a href='http://wiki'>Gurooo's wiki page</a></strong>";
    }
    document.getElementById("savedContacts").innerHTML = savedContactsContent;
    // Attach an event listener to the eventual link to restore the previous contacts
    if(document.getElementById("restoreContacts")) {
        document.getElementById("restoreContacts").addEventListener("click", clickRestoreContact, true);
    }
}

function checkOOO(savedContacts) {
    var now;
    
    // Check the date of the last OOO information
    now = new Date()
    if (GM_getValue("OOODate") != now.getYear() + "" + now.getMonth() + "" + now.getDate()) {
        OOOUpdatesSent = 0;
        OOOUpdatesReceived = 0;
        OOOStatus = [];
        // The OOO info is not up to date, get the info for each contact.
        for (var i in savedContacts){
            get(window.location.href + "Staff/Profile.aspx?id=" + savedContacts[i][0], updateOOO);
            OOOUpdatesSent++;
        }
        return false;
    }
    return true;
}

function updateOOO(responseText) {
    var id, now;
    
    if(responseText.indexOf("ctl00_Content_ctlProfile_EventList_pnlCurrentEvent")>-1) {
        // Contact is OOO
        id = responseText.split("Profile.aspx?id=")[1].split('"')[0];
        document.getElementById("OOO_" + id).style.display = "inline";
        OOOStatus.push(id);
    }
    OOOUpdatesReceived++;
    if (OOOUpdatesSent == OOOUpdatesReceived) {
        // All the OOO updates have been received, store all the values now
        now = new Date()
        GM_setValue("OOODate", now.getYear() + "" + now.getMonth() + "" + now.getDate());
        serialize("OOOStatus", OOOStatus);
    }
}

function clickRestoreContact(e) {
    e.preventDefault(); // Stop the click event on the link.
    GM_setValue("savedContacts", GM_getValue("savedContacts_backup"));
    GM_deleteValue("savedContacts_backup");
    
    // Re-populate the saved contacts section
    populateSavedContacts();
}

function clickRemoveContact(e) {
    var savedContacts, contactName;
    
    e.preventDefault(); // Stop the click event on the link.
    contactName = document.getElementById("ctl00_Content_ctlProfile_EmployeeIDCard_pnlCard_lblName").innerHTML;
    
    savedContacts = deserialize("savedContacts"); // Retrieve the already saved contacts
    delete savedContacts[contactName];
    serialize("savedContacts", savedContacts); // Store the contacts back
    
    remove(document.getElementById("contactStatus"));
    addSectionToProfile();
}

function clickSaveContact(e) {
    var contactID, contactName, contactExtension, contactEmail, savedContacts;
    
    // Retrieve the important information to save
    // The contact's ID (extracted from the url)
    contactID = window.location.href.split("/Staff/Profile.aspx?id=")[1];
    // The contact's Name
    contactName = document.getElementById("ctl00_Content_ctlProfile_EmployeeIDCard_pnlCard_lblName").innerHTML;
    // The contact's internal extension
    contactExtension = document.getElementById("ctl00_Content_ctlProfile_EmployeeIDCard_pnlCard_lblExtension").innerHTML;
    // The contact's email address
    contactEmail = document.getElementById("ctl00_Content_ctlProfile_EmployeeIDCard_pnlCard_lblEmail").href;
    
    // Add this user's info to the saved contacts array
    savedContacts = deserialize("savedContacts"); // Retrieve the already saved contacts
    savedContacts[contactName] = [contactID, contactExtension, contactEmail]; // Add the current contact
    serialize("savedContacts", sortContacts(savedContacts)); // Store the (sorted on names) contacts back
    
    // Update the UI to signify that this contact has been saved
    document.getElementById("contactStatus").innerHTML = contactName + " has been saved in your contact list!";
    document.getElementById("saveContact").style.display = "none";
    document.getElementById("contactStatus").style.display = "block";
    
    // Delete the cached OOO info, as we now have a new contact
    GM_deleteValue("OOODate");
}

function clickEmptyList(e) {
    e.preventDefault(); // Stop the click event on the link.
    document.getElementById("savedContacts").innerHTML = "Your list of saved contacts has been flushed!<br />Don't worry, you can <a href='' id='restoreContacts' style='display: inline; border: none; padding: 0px;'>undo this action</a>...";
    document.getElementById("restoreContacts").addEventListener("click", clickRestoreContact, true);
    document.getElementById("emptyListLink").style.display = "none";
    // Back up the data before deleting it, you never know...
    GM_setValue("savedContacts_backup", GM_getValue("savedContacts"));
    GM_deleteValue("savedContacts");
    GM_deleteValue("OOODate");
}

function sortContacts(savedContacts) {
    var indexArray, i, j, sortedSavedContacts;
    
    // Create an array that contains the names of all the contacts
    indexArray = [];
    for (i in savedContacts){
        indexArray.push(i);
    }
    
    // Sort that array (now we have the names of our contacts in alphabetical order)
    indexArray.sort();
    
    // Recreate and return the object with each contact sorted by name
    sortedSavedContacts = ({});
    for (j in indexArray){
        sortedSavedContacts[indexArray[j]] = savedContacts[indexArray[j]];
    }
    return sortedSavedContacts;
}

/************************
    Helper functions
************************/

// Adds an element at the beginning of a node
function insertAtBeginning(newNode, node) {
    node.insertBefore(newNode, nextSibling(node.firstChild));
}

// From http://www.dustindiaz.com/getelementsbyclass/
function getElementsByClass(searchClass,node,tag) {
    var classElements = new Array();
    if ( node == null )
        node = document;
    if ( tag == null )
        tag = '*';
    var els = node.getElementsByTagName(tag);
    var elsLen = els.length;
    var pattern = new RegExp("(^|\\s)"+searchClass+"(\\s|$)");
    for (i = 0, j = 0; i < elsLen; i++) {
        if ( pattern.test(els[i].className) ) {
            classElements[j] = els[i];
            j++;
        }
    }
    return classElements;
}

// From http://wiki.greasespot.net/Code_snippets#nextSibling_function_that_skips_whitespace
function nextSibling(startSib){
    if (!(nextSib=startSib.nextSibling)) {
        return false;
    }
    while (nextSib.nodeType!=1) {
        if (!(nextSib=nextSib.nextSibling)) {
            return false;
        }
    }
    return nextSib;
}

// From http://wiki.greasespot.net/Code_snippets#Remove_DOM_node
function remove(element) {
    element.parentNode.removeChild(element);
}

// From http://wiki.greasespot.net/Code_snippets#Build_a_DOM_node_with_attributes
function createElement(type, attributes){
    var node = document.createElement(type);
    for (var attr in attributes) {
        if (attributes.hasOwnProperty(attr)) {
            node.setAttribute(attr, attributes[attr]);
        }
    }
    return node;
}

// From http://wiki.greasespot.net/Code_snippets#Serialize.2Fdeserialize_for_GM_getValue
function deserialize(name, def) {
    return eval(GM_getValue(name, (def || '({})')));
}

function serialize(name, val) {
    GM_setValue(name, uneval(val));
}

// From http://wiki.greasespot.net/Code_snippets#GET_a_URL_with_callback_function
function get(url, cb) {
    GM_xmlhttpRequest({
        method: "GET",
        url: url,
        onload: function(xhr) { cb(xhr.responseText) }
    })
}

// From http://snook.ca/archives/javascript/testing_for_a_v/
function oc(a) {
    var o = {};
    for(var i=0;i<a.length;i++) {
        o[a[i]]='';
    }
    return o;
}

startProcess();
